@@ -2,8 +2,9 @@ import sys
 import os
 import datetime
 import pkg_resources
-import random
 import posixpath
+import tempfile
+import random
 
 import yaml
 from fabric.api import *
@@ -12,7 +13,7 @@ from fabric.contrib.files import exists, upload_template
 import silk.lib
 
 SRV_ROOT = '/srv'
-ROLLBACK_CAP = 3
+DEFAULT_ROLLBACK_CAP = 3
 
 def _get_silk_deps():
     silk_deps_file = pkg_resources.resource_filename('silk', 'deps.yaml')
@@ -40,9 +41,6 @@ def _set_vars():
     env.workdir = '/'.join(['/tmp', env.site])
     env.rollbackdir = '/'.join([SRV_ROOT, 'rollbacks'])
     env.deploytime = datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S')
-    #default to using a unix socket for nginx->gunicorn
-    if 'bind' not in env.config['gunicorn']:
-        env.config['gunicorn']['bind'] = 'unix:/tmp/%s.sock' % env.config['site']
     env.config['bind'] = env.config['gunicorn']['bind']
     env.config['silk_deps'] = _get_silk_deps()
     env.config['site_deps'] = _get_site_deps(env.local_root)
@@ -77,7 +75,8 @@ def _put_dir(local_dir, remote_dir):
 
 def _get_blame():
     """
-    Returns a yaml file that contains the site config, plus some deployment info
+    Returns a yaml file that contains the site config, plus some deployment
+    info.  The actual blame.yaml file will be written from this data.
     """
     blame = [
         {'deployed_by': env.user,
@@ -92,12 +91,10 @@ def _write_blame():
     """
     Writes blame file on remote host.
     """
-    #randomize the string to reduce chance of collision
-    blamefile = '/tmp/%s-blame.yaml' % ''.join([random.choice('abcdefghijklmnopqrstuvwxyz') for x in xrange(5)])
-    f = open(blamefile, 'w')
-    f.write(_get_blame())
-    f.close()
-    put(blamefile, '/'.join([env.workdir, 'blame.yaml']))
+    blamefile = tempfile.NamedTemporaryFile()
+    blamefile.write(_get_blame())
+    blamefile.seek(0) # Rewind the file so that the putter can read it.
+    put(blamefile.name, '/'.join([env.workdir, 'blame.yaml']))
     os.remove(blamefile)
 
 def reload():
@@ -120,32 +117,34 @@ def archive():
     """
     Creates rollback archive of already-deployed site.  Rotates old rollback files.
     """
-    print "CREATING ROLLBACK"
-    if not exists(env.rollbackdir, use_sudo=True):
-        sudo('mkdir -p %s' % env.rollbackdir)
+    ROLLBACK_CAP = env.config.get('rollback_cap', DEFAULT_ROLLBACK_CAP)
+    if ROLLBACK_CAP > 0:
+      print "CREATING ROLLBACK"
+      if not exists(env.rollbackdir, use_sudo=True):
+          sudo('mkdir -p %s' % env.rollbackdir)
 
-    template_vars = {
-        'rollback_cap': ROLLBACK_CAP,
-        'srv_root': SRV_ROOT,
-    }
+      template_vars = {
+          'rollback_cap': ROLLBACK_CAP,
+          'srv_root': SRV_ROOT,
+      }
 
-    template_vars.update(env)
+      template_vars.update(env)
 
-    oldest_rollback = '%(rollbackdir)s/%(site)s-rollback_%(rollback_cap)s.tar.bz2' % template_vars
-    #first delete the oldest rollback if present
-    if exists(oldest_rollback):
-        sudo('rm %s' % oldest_rollback) 
+      oldest_rollback = '%(rollbackdir)s/%(site)s-rollback_%(rollback_cap)s.tar.bz2' % template_vars
+      #first delete the oldest rollback if present
+      if exists(oldest_rollback):
+          sudo('rm %s' % oldest_rollback) 
 
-    #then increment the numbers on the existing rollbacks
-    for i in xrange(ROLLBACK_CAP - 1, 0, -1):
-        rollback_file = '%s/%s-rollback_%s.tar.bz2' % (env.rollbackdir, env.site, i)
-        if exists(rollback_file):
-            newname = '%s/%s-rollback_%s.tar.bz2' % (env.rollbackdir, env.site, i + 1)
-            sudo('mv %s %s' % (rollback_file, newname))
+      #then increment the numbers on the existing rollbacks
+      for i in xrange(ROLLBACK_CAP - 1, 0, -1):
+          rollback_file = '%s/%s-rollback_%s.tar.bz2' % (env.rollbackdir, env.site, i)
+          if exists(rollback_file):
+              newname = '%s/%s-rollback_%s.tar.bz2' % (env.rollbackdir, env.site, i + 1)
+              sudo('mv %s %s' % (rollback_file, newname))
 
-    #now archive env.remote_root if it exists
-    if exists(env.remote_root):
-        sudo('tar -cjf %(rollbackdir)s/%(site)s-rollback_1.tar.bz2 --exclude "*.log" -C %(srv_root)s %(site)s' % template_vars)
+      #now archive env.remote_root if it exists
+      if exists(env.remote_root):
+          sudo('tar -cjf %(rollbackdir)s/%(site)s-rollback_1.tar.bz2 --exclude "*.log" -C %(srv_root)s %(site)s' % template_vars)
 
 def rollback():
     """
@@ -210,6 +209,27 @@ def create_virtualenv():
     sudo('virtualenv --no-site-packages --python=%s %s' % (env.config['runtime'], env.envdir))
     build_mx_tools()
 
+def _pip_install_deps(dep_list):
+    # Write it out
+    file_contents = '\n'.join(dep_list)
+    tmpfile = tempfile.NamedTemporaryFile()
+    tmpfile.write(file_contents)
+    tmpfile.seek(0) # Rewind the file so that the putter can read it.
+
+    # Upload it to the remote host
+    tmp_remote = '/tmp/%s-reqs.txt' % ''.join([random.choice('abcdefghijklmnopqrstuvwxyz') for x in xrange(5)])
+    put(tmpfile.name, tmp_remote)
+
+    # Closing a tempfile will clean it up automatically.
+    tmpfile.close()
+
+    # Run pip install remotely
+    pip_cmd = silk.lib.get_pip_cmd(env.config) 
+    sudo('%s/bin/%s -r %s' % (env.envdir, pip_cmd, tmp_remote))
+         
+    # Clean up remote reqs file
+    sudo('rm %s' % tmp_remote)
+
 def install_python_deps():
     """
     Runs pip install inside the remote virtualenv.  First for silk's dependencies and
@@ -217,10 +237,7 @@ def install_python_deps():
     """
     for dep_dict in (env.config['silk_deps'], env.config['site_deps']):
         if dep_dict['python_packages']:
-            sudo('%s/bin/pip install %s' % (
-                env.envdir, 
-                ' '.join(dep_dict['python_packages']),
-            ))
+            _pip_install_deps(dep_dict['python_packages'])
 
 def install_deps():
     """
@@ -360,8 +377,8 @@ def cleanup():
 
 def server_setup():
     """
-    Installs nginx and supervisord on remote host.
-    Sets up nginx and supervisord global config files.
+    Installs nginx and supervisord on remote host.  Sets up nginx and
+    supervisord global config files.
     """
     install_apt_deps()
     template_list = (
@@ -373,9 +390,9 @@ def server_setup():
 
 def push():
     """
-    The main function.  Assuming you have nginx and supervisord installed
-    and configured, this function will put your site on the remote host and
-    get it running.
+    The main function.  Assuming you have nginx and supervisord installed and
+    configured, this function will put your site on the remote host and get it
+    running.
     """
     archive()
     install_deps()
